require 'pathname'
require  'find'
require "#{File.dirname(Pathname.new(__FILE__).realpath)}/gitlab"

InstallCodePathFile = File.expand_path('~')+'/.lzsource_path'

SplitWords = 'module_files'

module Pod
  class Command

    class Lzsource < Command

      $ProjectDir = 'unknow'
      $CodeAddr = 'unknow'
      $LibPath = 'unknow'
      $CodePath = 'unknow'
      $CloneFlag = false

      self.summary = 'Lookup the source code of the dependency library when debugging the project.'

      self.arguments = [
        CLAide::Argument.new('NAME', :optional)
      ]

      # self.arguments = [CLAide::Argument.new(%w(NAME), true)]

      def self.options
        [
        	['--list', 'Show all items list.'],
          ['--clear', 'Clear all items.'],
          ['--query', 'Query a item information.'],
          ['--reset', 'Reset token and gitlab url.'],
        ]
        # .concat(super)
      end

      def initialize(argv)
      	@list = argv.flag?('list')
      	@info = argv.flag?('query')
      	@clear = argv.flag?('clear')
        @reset = argv.flag?('reset')
        @name = argv.shift_argument
        super
      end

      def validate!
        super

        # help!
      end

      def run
      	if @clear != true &&  @list != true && @reset != true && @query != true && @name.nil?
      		help!
      		log()
      	end
      	if @clear == true
          checkPod()
      		clear()
      	elsif @list == true
          checkPod()
      		list()
        elsif @reset == true
          checkPod()
          reset()
      	elsif @query == true
      		if @name.nil?
      			help!
      			log()
      		else
            checkPod()
      			info(@name)
      		end
      	else

          checkPod()

          getSourceCodeInfo(@name) # CodeAddr ProjectDir

        	getLibPath() # LibPath

        	getDebugPath() # $CodePath
        	
          createDownloadDir()

          download()

          saveCodePath()

          cleanCache()

          list()

      	end
      end


      # 下载源代码流程 lzsource
      def getSourceCodeInfo(keyword)

        gitlabHttp= GitLabHttp.new 
        info = gitlabHttp.getSourceCodeInfo(keyword)
        if info.count > 0
          $CodeAddr = info[2]
          $ProjectDir = info[3]
        else
          UI.puts "No search result by the keyword.Please check the parameter."
          log()
        end

        if $ProjectDir == 'unknow'
          UI.puts 'Failed to get lib dir.'
          log()
        end
        if $CodeAddr == 'unknow'
          UI.puts 'Failed to get git address.'
          log()
        end

      end

      def getLibPath
        podPath = Dir.pwd+'/Pods/'
        path = Dir.pwd+'/Pods/'+$ProjectDir
        if File.directory?(podPath)
          if File.directory?(path)
           traverse_dir(path)
           if $LibPath == 'unknow'
            UI.puts 'Failed to get lib path.'
            log()
          end
         else
          UI.puts "Failed to get pod path.Please check the parameter."
          log()
         end
        else
          UI.puts "Failed to get pod path.This project directory may be not a cocoapods project."
          log()
        end
      end

      def traverse_dir(file_path)
        Find.find(file_path) do |filename|
          extn = File.extname filename
          if extn == '.a'
            $LibPath = filename
            break
          end
        end
      end

      def getDebugPath
        
        temp_dir = 'unknow'
        
        at_name = `dwarfdump -debug-str -arch=arm64 --lookup 0x0000002f #{$LibPath} | grep 'DW_AT_name' | head -1`
        arr = at_name.split('"')
        if arr.size > 1
          if arr[1].include?(SplitWords)
            temp = arr[1].split(SplitWords)
            if temp.size > 0
              temp_dir = temp[0]+SplitWords
            end
          end
        end

        temp_path = 'unknow'

        at_comp_dir = `dwarfdump -debug-str -arch=arm64 --lookup 0x0000002f #{$LibPath} | grep 'DW_AT_comp_dir' | head -1`
        arr = at_comp_dir.split('"')
        if arr.size > 1
          if arr[1].include?(SplitWords)
            temp = arr[1].split(SplitWords)
            if temp.size > 1
              temp_path = temp[1]
            end
          end
        end

        if temp_dir != 'unknow' && temp_path != 'unknow'
          $CodePath = temp_dir+temp_path
        else
          UI.puts 'Failed to get install path.'
          log()
        end
      end

      def createDownloadDir

        if File.directory?($CodePath)
          `sudo rm -rf #{$CodePath}`
        end

        `sudo mkdir -p -m 777 #{$CodePath};sudo chown -R $(whoami) #{$CodePath}`

        if !File.directory?($CodePath)
          UI.puts 'Failed to create install path.'
          log()
        end
      end

      def download
        t = Thread.new {
          startLoading()
        }
        cloneFlag = system("sudo git clone #{$CodeAddr} #{$CodePath} >/dev/null 2>&1")
        Thread.kill(t)
        if cloneFlag == false
          UI.puts 'Failed to download source code.'
          log()
        end
        print "\rDownload successful!\n"
      end

      def checkPod
          podPath = Dir.pwd+'/Pods/'
          if File.directory?(podPath)
          
          else
            UI.puts "Please check the current directory.This project directory may be not a cocoapods project."
            log()
          end
      end

      def saveCodePath
        f=File.open(InstallCodePathFile,"a+")
        f.puts $ProjectDir+"***"+$CodePath
        f.close
        
      end

      def cleanCache
        cachePath = File.expand_path('~')+"/Library/Developer/Xcode/DerivedData/"
        if File.directory?(cachePath)
          `rm -r #{cachePath} >/dev/null 2>&1`
        end
      end

      #lzsource --list
      def list
        if File.exist?(InstallCodePathFile)
          UI.puts "Currently installed:"
          tempNames = Array.new
          names = Array.new
          count = 0
          File.open(InstallCodePathFile,"r").each_line do |line|
            temp = line.gsub("\n", '')
            arr = temp.split("***")
            if !tempNames.include?(arr[0])
              names << " "+"#{count+1}:"+ arr[0]
              tempNames << arr[0]
              count = count +1
            end
          end
          puts names
        end
      end

      #lzsource --info
      def info(keyword)
        gitlabHttp= GitLabHttp.new 
        info = gitlabHttp.getProjectInfo(keyword)
        if info.count > 0
          puts '----------------'
        end
        info.each do |d|
          puts 'Repository name:'+d[0]
          puts 'Repository web:'+d[1]
          puts 'Repository ssh:'+d[2]
          # puts 'Repository path:'+d[3]
          puts '----------------'
        end
      end


      #lzsource --clear
      def clear
        if File.exist?(InstallCodePathFile)
            File.open(InstallCodePathFile,"r").each_line do |line|
              addr = line.gsub("\n", '')
              arr = addr.split("***")
              if File.directory?(arr[1])
                  `sudo rm -rf #{arr[1]}`
              end
            end
            File.delete(InstallCodePathFile)
        end
        UI.puts 'Clear successful!'
      end


      #lzsource --reset
      def reset
        gitlabHttp= GitLabHttp.new 
        gitlabHttp.resetInfo()
      end


      #tools

      def log
      	# UI.puts ''
       #  UI.puts 'Dir  Name：'+$ProjectDir
       #  UI.puts 'Lib  Path：'+$LibPath
       #  UI.puts 'Code Path：'+$CodePath
       #  UI.puts 'Code Addr：'+$CodeAddr
        exit
      end
      
      def startLoading
        spin = ["\\",'|','/','-']
        cunt = 0
        while true
          cunt +=1
          str = spin[cunt%4]
          print "downloading:#{str}"
          sleep 0.12
          print "\r"
        end
      end

    end
  end
end
